const nuxtConfig = require('../../nuxt.config')
const { Issuer, TokenSet } = require('openid-client')

// Configure openID-Connect client to verify tokens
// use the configuration that is supplied in ~/nuxt.config.js
const authStrategy = nuxtConfig.auth.strategies["oas"]
const openIdIssuer = new Issuer({
    authorization_endpoint: authStrategy.authorization_endpoint,
    token_endpoint: authStrategy.access_token_endpoint,
    userinfo_endpoint: authStrategy.userinfo_endpoint
});
var openIdClient;
(async () => {
  const openIdIssuer2 = await Issuer.discover(authStrategy.base_url);
  openIdClient = new openIdIssuer2.Client({
    client_id: authStrategy.client_id,
    client_secret: authStrategy.client_secret,
    redirect_uris: [authStrategy.redirect_uri],
    response_types: [authStrategy.response_type],
    grant_types: ["implicit"]
  })
})()


const authenticateWithToken = function (req, res, next) {
  // Get the oAuth2.0 access token from the user request
  // The User cookies are forwarded to this middleware
  // by nuxt add parsed by the cookieParser module which
  // makes the cookies accessible via req.cookies
  const authToken = req.cookies['auth._token.oas']
  const tokenSet = new TokenSet(
    {
      access_token: authToken.slice("Bearer ".length),
      token_type: "Bearer"
    })
  // We don't trust the token beacuse we received it via
  // the request object which is create by the client.
  // In order to veryfi the token, we query the userinfo
  // endpoint using the provided token. If the OpenID
  // provider acceppts the token, we know it's valid and
  // we will get the userinfo as at the same time
  openIdClient.userinfo(tokenSet).then(function(userinfo){
    // Safe our validated userinfo in the request object
    // for use in other express functions
    req.user = {
      name: userinfo.name,
      id: userinfo.sid,
      roles: userinfo[nuxtConfig.auth.scopeKey]
    }
    if ( userinfo.hasOwnProperty('preferred_username')){
      req.user.name = userinfo.preferred_username;
    }
    next()
  }).catch(function(error){
    res.status(403).send("Authentication failed: " + error)
  });
}

module.exports = authenticateWithToken
